##### Configuration

src_server ?= vegbiendev
src_user ?= $(USER)
test ?=

##### Vars/functions

# Make
null :=
space := $(null) $(null)
comma := ,
pathParts = $(shell path="$(1)"; echo "$${path%%/*}" "$${path\#*/}")
topDir = $(word 1,$(pathParts))
subPath = $(word 2,$(pathParts))
subMake = $(MAKE) $(call subPath,$@) --directory=$(call topDir,$@) \
--makefile=../input.Makefile
# input.Makefile path is relative to subdir

# Formatting
sed = sed -$(if $(filter Darwin,$(os)),E,r)
_2Space = $(subst _,$(space),$(1))
*2Space = $(call _2Space,$*)

# Filesystem
wildcard/ = $(shell bash -c 'shopt -s nullglob; echo $(1)')
    # needed because builtin $(wildcard) doesn't do / suffix correctly
no/ = $(1:/=)# remove trailing /
+w = $(wildcard $+)# existing items in $+
CP := cp -p
cp = $(CP) $< $@

# File editing
sudoAppend = echo $$'$(2)'|sudo tee -a $(1) >/dev/null

# System
os := $(shell uname)
forOs = $(patsubst %,%-$(filter Linux Darwin,$(os)),$(1))
date = $(shell date +"%-Y-%-m-%-d %-H:%M:%S %Z")

# Terminal
esc := '['
reset := $(esc)'0m'
emph := $(esc)'7m '
endEmph := ' '$(reset)

# User interaction

done = echo; echo $(emph)"Finished $@"$(endEmph); echo

wait := read -p $(emph)'Press ENTER to continue:'$(endEmph) REPLY

confirm = $(if $(shell read -p $(emph)"$(1)"$(endEmph)$$'$(if\
$(2),\n$(2))\nContinue? (y/n) ' REPLY; test "$$REPLY" = y && echo t),,\
$(error Aborting))

## SVN

addFile = $(if $(wildcard $(1)),,touch $(1) && )svn add $(1)
addDir = $(if $(wildcard $(1)/),svn add --depth=empty $(1),svn mkdir $(1))
add* = $(if $(1),svn add --depth=empty $(1))

# Revisions
revision = $(shell svn info $(1)|$(sed) -n 's/^Last Changed Rev: (.*)$$/\1/p')
rootRevision = $(call revision,$(root))
version ?= r$(rootRevision)

# rsync
rsync* := "time" rsync$(if $(test), --dry-run) --archive --update --verbose\
--itemize-changes --progress
rsync := $(rsync*) --exclude=".svn/" --exclude="*\#" --exclude=".DS_Store"
local := ./
localDirName := $(notdir $(realpath $(local)))
remote := $(src_user)@$(src_server):/home/bien/svn/$(localDirName)/

# DB
mkSchemaCmd = 'CREATE SCHEMA $(1);'
rmSchemaCmd = 'DROP SCHEMA IF EXISTS "$(1)" CASCADE;'

##### General targets

all: # must be first target in Makefile, so add an empty version here

.SUFFIXES: # turn off built-in suffix rules
.SECONDARY: # don't automatically delete intermediate files
.DELETE_ON_ERROR: # delete target if recipe fails

_always:
.PHONY: _always

clean: # make sure `make clean` always works

# Usage: `make {--silent|-s} version` (don't echo make commands)
version: _always
	echo $(version)
